import ssl
import pika
from django.conf import settings

pubsub_settings = {
    'username': settings.PUBSUB['RABBITMQ_USERNAME'],
    'password': settings.PUBSUB['RABBITMQ_PASSWORD'],
    'host': settings.PUBSUB['RABBITMQ_HOST'],
    'port': settings.PUBSUB['RABBITMQ_PORT'],
}


def get_parameters(host, port, username, password):
    credentials = pika.PlainCredentials(username, password)
    context = get_context()
    ssl_options = pika.SSLOptions(context)
    parameters = pika.ConnectionParameters(
        host=host,
        port=port,
        credentials=credentials,
        ssl_options=ssl_options,
    )
    return parameters


def get_context():
    context = ssl.create_default_context()
    context.check_hostname = False
    context.verify_mode = ssl.CERT_NONE
    return context



import pika
import json
from .pubsub import get_parameters, pubsub_settings


class Producer:
    def __init__(self, host, username, password, port=5671):
        self.parameters = get_parameters(host, port, username, password)
        self.exchanges = []

    def produce(self, exchange, body, routing_key=''):
        connection = pika.BlockingConnection(self.parameters)
        channel = connection.channel()
        if exchange not in self.exchanges:
            channel.exchange_declare(exchange=exchange, exchange_type='direct')
            self.exchanges.append(exchange)

        channel.basic_publish(
            exchange=exchange,
            routing_key=routing_key,
            body=json.dumps(body),
            properties=pika.BasicProperties(
                delivery_mode=2,  # make message persistent
            )
        )
        connection.close()

    @classmethod
    def get_producer(cls):
        return cls(**pubsub_settings)



import pika
from .pubsub import get_parameters, pubsub_settings


class Consumer:
    def __init__(self, host, username, password, port=5672):
        self.parameters = get_parameters(host, port, username, password)
        self.connection = None

    def consume(self, exchange, queue_name, routing_key, callback):
        if not self.connection:
            self._init_connection()
        self._init_channel()
        queue_name = self._init_queue(exchange, queue_name, routing_key)
        self.channel.basic_qos(prefetch_count=1)
        self.channel.basic_consume(
            queue=queue_name,
            on_message_callback=callback,
        )
        print(' [*] Waiting for messages. To exit press CTRL+C')
        self.channel.start_consuming()

    def _init_connection(self):
        self.connection = pika.BlockingConnection(self.parameters)

    def _init_channel(self):
        self.channel = self.connection.channel()

    def _init_queue(self, exchange, queue_name, routing_key):
        """declare and bind queue"""
        result = self.channel.queue_declare(queue=queue_name, durable=True)
        self.channel.queue_bind(
            exchange=exchange,
            queue=result.method.queue,
            routing_key=routing_key
        )
        return result.method.queue

    @classmethod
    def get_consumer(cls):
        return cls(**pubsub_settings)
